"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClozeRegExp = void 0;
const ClozeFieldEnum_1 = require("./ClozeFieldEnum");
class ClozeRegExp {
    constructor(pattern, clozeFieldsOrder, flags) {
        this.regex = new RegExp(pattern, flags);
        this.clozeFieldsOrder = clozeFieldsOrder;
    }
    exec(str) {
        let match = this.regex.exec(str);
        if (!match) {
            return null;
        }
        // All clozes (Cloze simple, Cloze Classic and Cloze overlapping) have an answer and can have a hint
        if (this.clozeFieldsOrder.indexOf(ClozeFieldEnum_1.ClozeFieldEnum.answer) == -1) {
            throw new Error("Cloze text not found in clozeFieldsOrder");
        }
        if (this.clozeFieldsOrder.indexOf(ClozeFieldEnum_1.ClozeFieldEnum.hint) == -1) {
            throw new Error("Cloze hint not found in clozeFieldsOrder");
        }
        // But cloze simple doesn't have cloze seq
        if (this.clozeFieldsOrder.indexOf(ClozeFieldEnum_1.ClozeFieldEnum.seq) == -1) {
            match.seq = null;
        }
        else {
            match.seq = match[this.clozeFieldsOrder.indexOf(ClozeFieldEnum_1.ClozeFieldEnum.seq) + 1];
        }
        match.raw = match[0];
        match.answer = match[this.clozeFieldsOrder.indexOf(ClozeFieldEnum_1.ClozeFieldEnum.answer) + 1];
        match.hint = match[this.clozeFieldsOrder.indexOf(ClozeFieldEnum_1.ClozeFieldEnum.hint) + 1];
        return match;
    }
    test(str) {
        return this.regex.test(str);
    }
}
exports.ClozeRegExp = ClozeRegExp;
